.include  "8515def.inc"


; Baud Rate fr Midi
.EQU	BAUDRC =7		; for 4Mhz clock

; Startadresse fr SRAM
.EQU	START	=0x0400

; Timer 1 Prescaler
.EQU	T1SCL	=0x05

; Midi Befehle
.EQU	GMreset	=255
.EQU	MIDIalv	=0xFE
.EQU	MIDIclk	=0xF8
.EQU	MIDIstop=0xFC

; Pinbelegung
; PORTB
.EQU	KEY1	=0	; play btn
.EQU	KEY2	=1	; rec btn
.EQU	LED1	=2
.EQU	LED2	=3
.EQU	LED3	=4
.EQU	LEDp	=5
.EQU	LEDr	=6
.EQU	KEY3	=7	; stop btn
; PORTD
.EQU	RxD	=0
.EQU	TxD	=1
.EQU	A16	=2
.EQU	A17	=3
.EQU	A18	=4
.EQU	KEY4	=5	; file btn
.EQU	WR	=6
.EQU	RD	=7
; PORT A, C = SRAM

; Modi
.EQU	smPLAY	=0x01
.EQU	mPLAY	=0
.EQU	smREC	=0x02
.EQU	mREC	=1
.EQU	smFULL	=0x04
.EQU	mFULL	=3
.EQU	smEOS	=0x08
.EQU	mEOS	=4



.CSEG

.DEF	NULL	=r0		; always handy to speed up carry operations
.DEF	BS	=r7

.DEF	greg	=r16 		; * watch: not used beyond initialization
.DEF	inreg	=r17		; UART RX 
.DEF	work	=r18 		; working register
.DEF	oldkey	=r5		; previous key press data
.DEF	keys	=r20
.DEF	mode	=r21
.DEF	t0	=r22
.DEF	t1	=r25
.DEF	t2	=r19
.DEF	t1alt	=r8

.DEF	p0	=r2
.DEF	p1	=r3
.DEF	p2	=r4

.DEF	uadr	=r24
.DEF	regs	=r1
.DEF	sd	=r23
.DEF	temp	=r6


.EQU	RESET=0


; *** Interupt vectors/routines

.ORG	RESET				; *** reset vector 
	rjmp main

;.ORG	OVF1addr
;	rjmp timer1

.ORG	URXCaddr			; *** Uart Interupt vector
	rjmp fifoin

.ORG	UTXCaddr
	rjmp sendcp



fifoin:	in 	inreg, 	UDR		; fetch byte from UART data reg. 
	in 	regs, 	SREG
	sbr 	greg, 	0x01
	out 	SREG, 	regs
	reti


timer1:	in 	regs, 	SREG
	inc 	t2
	out 	SREG, 	regs
	reti


sendcp:	in 	regs, 	SREG
	sbr 	greg, 	0x04
	out 	SREG, 	regs
	reti


; *** Main program

	; ***Initialization
main:
	clr 	mode
	clr	uadr
	clr 	greg
	clr	NULL
	ldi	work,	255
	mov	BS,	work

	ldi 	work, 	0x7F ;LOW(RAMEND)	; set up stack
	out 	SPL, 	work
	ldi 	work, 	0 ;HIGH(RAMEND)
	out 	SPH, 	work					 

					; ***UART setup
	ldi 	work, 	BAUDRC		; baudrate constant
	out 	UBRR, 	work		; prepare baud rate register
	sbi 	UCR, 	RXEN		; set "receive enable flag"
	sbi 	UCR, 	TXEN		; set "transmitter enable flag"
;	sbi 	UCR, 	RXCIE		; and "received character interrupt enable flag"
	sbi 	UCR, 	TXCIE		; and "received character interrupt enable flag"

					; ***Timer setup
	out 	TCCR1B,	NULL		; 0x02 to start timer
	out	TCNT1H,	NULL
	out 	TCNT1L,	NULL		; Timer 0.5ms

					; ***Port setup
	sbi	DDRB,	LED1		; LEDs Ausgnge
	sbi	DDRB,	LED2
	sbi	DDRB,	LED3
	sbi	DDRB,	LEDp
	sbi	DDRB,	LEDr
	ldi	work,	0xFF		; Pullups
	out	PORTB,	work

	sbi	DDRD,	A16		; Adressleitungen
	sbi	DDRD,	A17		; Adressleitungen
	sbi	DDRD,	A18		; Adressleitungen
	sbi	DDRD,	RD
	sbi	DDRD,	WR
	out	PORTD,	NULL
	sbi	PORTD,	KEY4

; ***FIFO initialization

fifoini:in 	work, 	UDR		; dummy read 
	clt
	; Enable external SRAM
	ldi	work,	(1<<SRE)	;|(1<<SRW)
	out	MCUCR,	work

	ldi 	work,	1<<TICIE1
	out 	TIMSK,	work		; Timer interrupt
	sei

; *** Main Program loop

; DEBUG
;  db0:
;	clr	greg
 ; db1:	cpi	greg,	1
;	brne	db1
;
;	cpi	inreg,	MIDIalv
;	breq	db2
;	rjmp	db0
 ; db2:
  ;	in	work,	PINB
;	com	work
;	out	PORTB,	work
 ; 	rjmp	db0
rjmp wff

	ldi	YL,	0		; Schreiber
	ldi	YH,	4
	ldi	XL,	0		; Leser
	ldi	XH,	4

	ldi	work,	0xAA
 db1:	st	Y+,	work
	cpi	YH,	2
	brne	db1

 db2:	ld	work,	X+
	cpi	work,	0xAA
	brne	ERR
	cpi	XH,	2
	brne	db2

 psok:	cbi	PORTB,	LEDp

	ldi	YL,	0		; Schreiber
	ldi	YH,	4
	ldi	XL,	0		; Leser
	ldi	XH,	4

	ldi	work,	0xFF
 db3:	st	Y+,	work
	cpi	YH,	254
	brne	db3

 db4:	ld	work,	X+
	cpi	work,	0xFF
	brne	ERR
	cpi	XH,	254
	brne	db4

 ps1ok:	cbi	PORTB,	LED1
	
	ldi	YL,	0		; Schreiber
	ldi	YH,	4
	ldi	XL,	0		; Leser
	ldi	XH,	4

	ldi	work,	0x00
 db5:	st	Y+,	work
	cpi	YH,	254
	brne	db5

 db6:	ld	work,	X+
	cpi	work,	0x00
	brne	ERR
	cpi	XH,	254
	brne	db6

 ps2ok:	cbi	PORTB,	LED2

	ldi	YL,	0		; Schreiber
	ldi	YH,	4
	ldi	XL,	0		; Leser
	ldi	XH,	4

	ldi	work,	0xAA
 db7:	st	Y+,	work
	cpi	YH,	254
	brne	db7

 db8:	ld	work,	X+
	cpi	work,	0xAA
	brne	ERR
	cpi	XH,	254
	brne	db8

 ps3ok:	cbi	PORTB,	LED3


	t:rjmp t

 ERR:	cbi	PORTB,	LEDr
	t2:rjmp t2
; /DEBUG


WFF:	in 	keys, 	PINB		; key check B
	andi	keys,	(1<<KEY1)|(1<<KEY2)|(1<<KEY3)
	in	work,	PIND		; key check D
	andi	work,	(1<<KEY4)
	or	keys,	work		; Keys loaded
	cp 	keys, 	oldkey
	mov 	oldkey,	keys
	brne 	KEY
  res:
	sbrc	mode,	mPLAY
	  rcall	PLAY1
	sbrc	mode,	mREC
	  rcall	recmidi
	rjmp 	WFF



; *** PLAY Polling

PLAY1:	sbrs	mode,	mPLAY		; Check Modus Play
	  ret
	sbrc	mode,	mREC		; Check Modus Play
	  ret
	;Timer t2 check
	;rcall	compT1
	;Vergleiche t (Timerstand) mit p (Liedposition)
	in	t0,	TCNT1L
	in	t1,	TCNT1H
	cp	t2,	p2
	breq	nchk1
	brlo	pexit
	rjmp	posok
 nchk1:	cp	t1,	p1
 	breq	nchk2
	brlo	pexit
	rjmp	posok
 nchk2:	cp	t0,	p0
	brlo	pexit
	; Pos. passt: Gebe Daten wieder, lade neue Daten
 posok:	out	UDR,	sd

 wplay:	sbrs	greg,	2
	  rjmp	wplay
	cbr	greg,	0x04

	rcall	readmidi
 pexit:	ret



; *** Tastenbehandlung

KEY:	cpi 	keys, 	(1<<KEY2)|(1<<KEY4)
	breq 	TESTzw			; key1+3 = Test Midi
	cpi 	keys, 	(1<<KEY1)|(1<<KEY2)|(1<<KEY3)|(1<<KEY4)
	breq 	res			; if no key is pressed
	cpi 	keys, 	(1<<KEY2)|(1<<KEY3)|(1<<KEY4)
	breq 	PLAY			; key1 = PLAY
	cpi 	keys, 	(1<<KEY1)|(1<<KEY3)|(1<<KEY4)
	breq 	REC			; key2 = REC
	cpi 	keys, 	(1<<KEY2)|(1<<KEY1)|(1<<KEY4)
	breq 	STOP			; key3 = STOP
	cpi 	keys, 	(1<<KEY2)|(1<<KEY3)|(1<<KEY1)
	breq 	INCS			; key4 = INC SongNo.
	rjmp 	res


; *** PLAY Taste

PLAY:	sbrc	mode,	mREC
	  rjmp	res
	sbrc	mode,	mPLAY
	  rjmp	res
	; Modus Play
	sbr 	mode, 	smPLAY
	; Pointer auf song start
	rcall	initpnt
	; Lies ersten Datensatz
	rcall	readmidi
	; Timer start
	ldi 	work, 	T1SCL
	out 	TCCR1B,	work
	cbi	PORTB,	LEDp
	rjmp	res


; *** REC Taste

REC:	sbrc	mode,	mPLAY
	  rjmp	res
	sbrc	mode,	mREC
	  rjmp	res
	; Modus Record
	sbr 	mode, 	smREC
	; Zhler reset
	rcall	initpnt
	in	work,	UDR
	sbi 	UCR, 	RXCIE
	cbi	PORTB,	LEDr
	rjmp	res


; *** STOP Taste

STOP:	; Stoppt REC und PLAY
	out	TCCR1B,	NULL
	; Wenn im Rec Mode, schreibe STOPP Event und RxInt Disable
	sbrc	mode,	mREC
	  rcall	writestop
	sbrc	mode,	mREC
	  cbi 	UCR, 	RXCIE
	clr	mode
	; Midigert ausmachen
;	rcall	sendRESET
	sbi	PORTB,	LEDr
	sbi	PORTB,	LEDp
	rjmp	res



testzw:	rjmp	test


; *** INCS Taste

INCS:	; Zhlt Songnummer hoch
	sbrc	mode,	mPLAY		; Check Modus Play
	  rjmp	res
	sbrc	mode,	mREC		; Check Modus Rec
	  rjmp	res
	inc	uadr
	cpi	uadr,	8
	brne	i2
	clr	uadr
  i2:	rcall	initpnt
  	rjmp	res


; *** TEST Tasten

TEST:	clr	greg
	ldi	work,	0x94
	out	UDR,	work
  tt1:	cpi	greg,	4
	brne	tt1
	clr	greg
	ldi	work,	0x40
	out	UDR,	work
  tt2:	cpi	greg,	4
	brne	tt2
	clr	greg
	ldi	work,	0x60
	out	UDR,	work
  tt3:	cpi	greg,	4
	brne	tt3
	clr	greg
	ldi	YL,	LOW(START)
	ldi	YH,	HIGH(START)

	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	0x10
	st	Y+,	work
	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	0x84
	st	Y+,	work

	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	0x10
	st	Y+,	work
	ldi	work,	0x02
	st	Y+,	work
	ldi	work,	0x40
	st	Y+,	work

	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	0x10
	st	Y+,	work
	ldi	work,	0x04
	st	Y+,	work
	ldi	work,	0x60
	st	Y+,	work

	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	0x11
	st	Y+,	work
	ldi	work,	0x00
	st	Y+,	work
	ldi	work,	MIDIstop
	st	Y+,	work

	rjmp	PLAY






; *** Zusatzfunktionen

;compT1:	; Vergleicht T1neu mit T1alt nur HB
;	cp	t1,	t1alt
;	brsh	excp
;	inc	t2
; excp:	mov	t1alt,	t1
;	ret


writestop:; Schreibt Stoppevent
	ldi	inreg,	MIDIstop
	rcall	writemidi
	ret


sendRESET:; Setzt MIDI Gert zurck
	ldi	work,	GMreset
	out	UDR,	work
	cbr	greg,	0x04
  wfs:	sbrs	greg,	2
	  rjmp	wfs
	ret


recmidi:; Zeichnet ein Midievent auf
	sbrs	greg,	0	; Nur wenn ein Zeichen gekommen ist
	  ret
	cbr	greg,	0x01
	;Timer t2 check
	;rcall	compT1
	; Prfe Daten
	cpi	inreg,	MIDIclk
	breq	exr
	cpi	inreg,	MIDIalv
	breq	exr
	; Timer start
	in	work,	TCCR1B
	andi	work,	7
	cpi	work,	T1SCL
	breq	rm1
	ldi 	work, 	T1SCL
	out 	TCCR1B,	work
	; Schreibe Daten
  rm1:	rcall	writemidi
	cbi	PORTB,	LEDp
  exr:	ret


readmidi:; Liest einen Datensatz ein
	rcall	rdchar
	mov	p2,	work
	rcall	rdchar
	mov	p1,	work
	rcall	rdchar
	mov	p0,	work
	rcall	rdchar
	mov	sd,	work
	cpi	work,	MIDIstop
	breq	h_eos
	sbrc	mode,	mFULL		; Voll?
	  rjmp	h_full
	ret
 h_eos:	; Ende vom Lied
	out	TCCR1B,	NULL
	clr	mode
	; Midigert ausmachen
;	rcall	sendRESET
	sbi	PORTB,	LEDp
	ret


writemidi:; Speichert Miditiming und Byte im SRAM
	in	t0,	TCNT1L
	in	t1,	TCNT1H
	mov 	work, 	t2
	rcall 	wrtchar
	mov 	work, 	t1
	rcall 	wrtchar
	mov 	work, 	t0
	rcall 	wrtchar
	mov 	work, 	inreg
	rcall 	wrtchar
	sbrc	mode,	mFULL		; Voll?
	  rjmp	h_full
	ret
 h_full:; Speicher fr Song voll!
	out	TCCR1B,	NULL
	clr	mode
	; Midigert ausmachen
;	rcall	sendRESET
	sbi	PORTB,	LEDp
	ret


wrtchar:; Speichert 1 Byte im SRAM
	st	Y+,	work
;	nop
;	ld	temp,	Y
;	cp	temp,	work
;	brne	wrtchar
;	ld	temp,	Y+
	cpi	YH,	0
	brne	nadr
	sbr	mode,	smFULL
  nadr:	ret


rdchar:; Liest 1 Byte aus SRAM
;	ld	work,	Y
;	mov	temp,	work
;	nop
;	ld	temp,	Y
;	cp	temp,	work
;	brne	rdchar
	ld	work,	Y+
	cpi	YH,	0
	brne	nadr2
	sbr	mode,	smFULL
 nadr2:	ret


initpnt:; Setzt Y Lese und Schreibzeiger zurck
	ldi	YL,	LOW(START)
	ldi	YH,	HIGH(START)
	; Zhler reset
	clr 	t2
	clr	t1alt
	cbr	mode,	smFULL
	cbr	mode,	smEOS
	; Timer 1 Reset
	out	TCCR1B,	NULL
	out	TCNT1H,	NULL
	out	TCNT1L,	NULL
	; Ldt Seiten-Adresse
	mov	work,	uadr
	clc
	rol	work
	clc
	rol	work
	ori	work,	(1<<KEY4)
	out	PORTD,	work
	; LED Anzeige
	com	work			
	ori	work,	(1<<KEY1)|(1<<KEY2)|(1<<KEY3)|(1<<LEDr)|(1<<LEDp)
	out	PORTB,	work
	ret



